/****************************************************************************
 *		 Fitness Evaluation                                         *
 ****************************************************************************/

#include "defs.h"
#include <stdlib.h>
#include <math.h>

#include <iostream>
#include <string>
#include <fstream>

using namespace std;

/****************************************************************************
 *			Calculate Fitness - Royal Road 			    *
 ****************************************************************************/
double calcFitness_royal(int *indiv) {
  int product, i, j;
  double fitness=0.0;

  for (i=0; i<8; i++) {
    product = 1;
    for (j=i*8; j<i*8+8; j++) 
      product *= indiv[j];

    fitness += (double)8.0*product;
  }

  return fitness;
}

/******************************************************/
/* Decomposable Unitation-based Functions (DUFs)      */
/******************************************************/
double calcFitness_DUF(int *indiv) {
  int i, j, countOFOnes, fitness = 0;
  int NBLOCKS = 25;    // the number of schemas
  int BLOCKSIZE = 4;   // block size - length of target schema

  for (i = 0; i < NBLOCKS; ++i) { 
    countOFOnes=0;
    for (j = 0; j < BLOCKSIZE; j++)
      if (indiv[i*BLOCKSIZE + j] == 1)
        countOFOnes++;  // count the 1 bits in the block

    switch(functNo) {
      case 1: // DUF(25,4,1)
        fitness += countOFOnes;
        break;
      case 2: // DUF(25,4,2)
        if (countOFOnes == 4) fitness += 4;
        else if(countOFOnes == 3) fitness += 2;
        break;
      case 3: // DUF(25,4,4)
        if (countOFOnes == 4) fitness += 4;
        break;
      case 4: // DUF(25,4,deceptive)
        if (countOFOnes == 4) fitness += 4;
        else fitness += (3 - countOFOnes);
        break;
    }
  }

  return (double)fitness;
}

/*************************************************************/
/* Knapsack Problem: select items to fill a knapsack to      */
/*    maximize profits, defined as follows:                  */
/*                                                           */
/*    f(X)= sum_i (p_i*x_i)                                  */
/*    subject to:                                            */
/*        sum_i (w_i*x_i) <= CAPACITY                        */
/*                                                           */
/*************************************************************/
void initializeKnapsack(void) {
  int totalW = 0;
  ifstream ifs("Knapsack100.txt",ios::in);
  if (!ifs) {
    cerr << "Error: unable to open the data file." << endl;
    exit(1);
  }

  char temp[20];
  for (int i = 0; i < lchrom; i++) {
    ifs >> temp >> weight[i] >> temp >> profit[i];
    totalW += weight[i];
  }
  ifs.close();

  TOTALWEIGHT = totalW;
  CAPACITY = (int)(capFactor * TOTALWEIGHT);
}

double calcFitness_Knapsack(int *indiv) {
  double sumW, sumP;
  double pFactor = 0.00001;   // punish factor
  double fitness = 0.0;

  sumW = sumP = 0.0;
  for (int i = 0; i < lchrom; i++) {
    if (indiv[i] == 1) {
      sumW += weight[i];
      sumP += profit[i];
    }
  }

  if (sumW > CAPACITY)
    fitness = pFactor *(TOTALWEIGHT - sumW);
  else fitness = sumP;

  return fitness; 
}

/***************************************************************************
 *			Calculate Fitness 				   *
 ***************************************************************************/
double calcFitness(int *indiv) {
  double fitness;
  int *tmpIndiv = new int[lchrom];
  for (int i=0; i<lchrom; i++)
    tmpIndiv[i] = indiv[i]^xorMask[i];

  if (functNo == 5)
    fitness = calcFitness_Knapsack(tmpIndiv);
  else fitness = calcFitness_DUF(tmpIndiv);

  delete [] tmpIndiv;

  return fitness;
}
